import numpy as np
import pandas as pd
from math import sqrt
import os
import datetime

# 百分数脱敏
def ratio_tuomin(x):
    # x是百分比的数字部分,即比例*100,比如是54.3%的,即为54.3,要把它脱敏变成55
    # 精度这里我建议我们按照2、5、7、10 进行分割，选择绝对止值距离较近的那一档，举个🌰，
    # 1.13% ➡️12%、9.12% ➡️10%、43.53% ➡️45%。你看看可以吗～
    # 非个位数的
    def mymap(x):
        if 0 <= x < 1: x = 0
        elif 1 <= x < 3.5: x = 2
        elif 3.5 <= x < 6: x = 5
        elif 6 <= x < 8.5: x = 7
        else : x = 10
        return x
    flag = -1 if x < 0 else 1
    x = abs(x)
    if x > 10:
        first_num = x // 10
        second_num = x % 10 
        return flag * (first_num * 10 + mymap(second_num))
    else:
        return flag * mymap(x)

# 根据起始、终止时间切分df表的，返回的是历史时期某一列的nparray数组
def get_his_y_values(mydata,x,y,time_begin,time_end):
    y_his = mydata[ (mydata[ x ] >= time_begin) & (mydata[ x ] <= time_end) ][ y ].values
    return y_his

# 这个是算两个列表绝对差异的 返回的第一个值是平均绝对差异，第二个值是最大绝对差异
def gap_func(list1,list2):
    n1 = len(list1)
    n2 = len(list2)
    # 一般起点一样终点不一样，切掉后面不一样的
    if n1 > n2:
        list1 = list1[:n2]
    else:
        list2 = list2[:n1]
    mygap_tmp = []
    for i in range(len(list1)):
        if list2[i] > 0:
            if abs((list1[i] - list2[i])/list2[i]) < 100:
                mygap_tmp.append((list1[i] - list2[i])/list2[i])
    # 计算绝对差异
    mygap = abs(np.array(mygap_tmp)).mean()
    # 计算极端差异
    try:
        mygap_max = abs(np.array(mygap_tmp)).max()
    except:
        print("极端差异计算报错，记为负1")
        mygap_max = -1
    return mygap,mygap_max

def getYesterday(): 
    today = datetime.date.today() 
    oneday = datetime.timedelta(days=1) 
    yesterday = today-oneday  
    tmp = str(yesterday)
    ans = tmp[:4] + tmp[5:7] + tmp[8:10]
    return ans


def get_holiday_qujian_date(myholiday_name):
    if myholiday_name == "同期国庆":
        h_begin = np.datetime64(datetime.datetime.strptime(str('2020-09-25'),'%Y-%m-%d').date())
        h_end   = np.datetime64(datetime.datetime.strptime(str('2020-10-10'),'%Y-%m-%d').date())
        a_begin = np.datetime64(datetime.datetime.strptime(str('2020-10-01'),'%Y-%m-%d').date())
        a_end   = np.datetime64(datetime.datetime.strptime(str('2020-10-07'),'%Y-%m-%d').date())
        b_begin = np.datetime64(datetime.datetime.strptime(str('2020-09-24'),'%Y-%m-%d').date())
        b_end   = np.datetime64(datetime.datetime.strptime(str('2020-09-30'),'%Y-%m-%d').date())
    if myholiday_name == "同期五一":
        h_begin = np.datetime64(datetime.datetime.strptime(str('2020-04-24'),'%Y-%m-%d').date())
        h_end   = np.datetime64(datetime.datetime.strptime(str('2020-05-09'),'%Y-%m-%d').date())
        a_begin = np.datetime64(datetime.datetime.strptime(str('2020-05-01'),'%Y-%m-%d').date())
        a_end   = np.datetime64(datetime.datetime.strptime(str('2020-05-05'),'%Y-%m-%d').date())
        b_begin = np.datetime64(datetime.datetime.strptime(str('2020-04-24'),'%Y-%m-%d').date())
        b_end   = np.datetime64(datetime.datetime.strptime(str('2020-04-30'),'%Y-%m-%d').date())
    if myholiday_name == "同期元旦":
        h_begin = np.datetime64(datetime.datetime.strptime(str('2020-12-24'),'%Y-%m-%d').date())
        h_end   = np.datetime64(datetime.datetime.strptime(str('2021-01-09'),'%Y-%m-%d').date())
        a_begin = np.datetime64(datetime.datetime.strptime(str('2021-01-01'),'%Y-%m-%d').date())
        a_end   = np.datetime64(datetime.datetime.strptime(str('2021-01-03'),'%Y-%m-%d').date())
        b_begin = np.datetime64(datetime.datetime.strptime(str('2020-12-25'),'%Y-%m-%d').date())
        b_end   = np.datetime64(datetime.datetime.strptime(str('2020-12-31'),'%Y-%m-%d').date())
    if myholiday_name == "同期618":
        h_begin = np.datetime64(datetime.datetime.strptime(str('2020-06-01'),'%Y-%m-%d').date())
        h_end   = np.datetime64(datetime.datetime.strptime(str('2020-06-20'),'%Y-%m-%d').date())
        a_begin = np.datetime64(datetime.datetime.strptime(str('2020-06-16'),'%Y-%m-%d').date())
        a_end   = np.datetime64(datetime.datetime.strptime(str('2020-06-18'),'%Y-%m-%d').date())
        b_begin = np.datetime64(datetime.datetime.strptime(str('2020-06-09'),'%Y-%m-%d').date())
        b_end   = np.datetime64(datetime.datetime.strptime(str('2020-06-15'),'%Y-%m-%d').date())
    if myholiday_name == "同期双11":
        h_begin = np.datetime64(datetime.datetime.strptime(str('2020-11-01'),'%Y-%m-%d').date())
        h_end   = np.datetime64(datetime.datetime.strptime(str('2020-11-20'),'%Y-%m-%d').date())
        a_begin = np.datetime64(datetime.datetime.strptime(str('2020-11-09'),'%Y-%m-%d').date())
        a_end   = np.datetime64(datetime.datetime.strptime(str('2020-11-11'),'%Y-%m-%d').date())
        b_begin = np.datetime64(datetime.datetime.strptime(str('2020-11-04'),'%Y-%m-%d').date())
        b_end   = np.datetime64(datetime.datetime.strptime(str('2020-11-10'),'%Y-%m-%d').date())
    if myholiday_name == "同期春节":
        h_begin = np.datetime64(datetime.datetime.strptime(str('2021-01-20'),'%Y-%m-%d').date())
        h_end   = np.datetime64(datetime.datetime.strptime(str('2021-02-25'),'%Y-%m-%d').date())
        a_begin = np.datetime64(datetime.datetime.strptime(str('2020-05-01'),'%Y-%m-%d').date())
        a_end   = np.datetime64(datetime.datetime.strptime(str('2020-05-05'),'%Y-%m-%d').date())
        b_begin = np.datetime64(datetime.datetime.strptime(str('2020-04-24'),'%Y-%m-%d').date())
        b_end   = np.datetime64(datetime.datetime.strptime(str('2020-04-30'),'%Y-%m-%d').date())
    return h_begin,h_end,a_begin,a_end,b_begin,b_end



# -*- coding:UTF-8 -*-
# pyinstaller -F -w biqukan/biqukan.py
from bs4 import BeautifulSoup
import requests, sys
import json
import tkinter 
import tkinter.ttk
import threading
import time
import re
import os
"""
类说明:下载《笔趣看》网小说各类小说
Parameters:
	无
Returns:
	无
Modify:
	2017-09-13
"""

s = requests.session()
s.keep_alive = False

# 建立目录
def mkdir(directory):
	if not os.path.exists(directory):
		os.makedirs(directory)

class nov_dl(object):
	def __init__(self):
		self.server = r'http://www.biqukan.com/'
		'''
		with open('biqukan/url.json', 'r', encoding = 'utf-8') as f:
			self.url_data = json.load(f)
		'''
		self.url_data = dict()
		self.book_name = ""
		self.save_path = r"G:\\nextcloud\\小说\\爬虫下载\\"
		#self.save_path = "/Users/qiuqiandong/Documents/mycode/novel/"
		self.target = []
		self.names = []			#存放章节名
		self.urls = []			#存放章节链接
		self.nums = 0			#章节数
	"""
	函数说明:获取下载链接
	Parameters:
		无
	Returns:
		无
	Modify:
		2017-09-13
	"""
	def get_download_url(self):
		headers = { 'Accept':'application/json, text/plain, */*', 'Accept-Language':'zh-CN,zh;q=0.8', 'User-Agent':'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36' }
		proxies={
			'http':'127.0.0.1:8080'
			}
		req = requests.get(
			url = self.target,
			headers=headers,
			#proxies = proxies,
			#verify=False
			)
		req.encoding = "gbk"
		html = req.text
		div_bf = BeautifulSoup(html)
		div = div_bf.find_all('div', class_ = 'listmain')
		a_bf = BeautifulSoup(str(div[0]))
		a = a_bf.find_all('a')
		self.nums = len(a[12:])								
		#剔除不必要的章节，并统计章节数
		for each in a[12:]:
			self.names.append(each.string)
			self.urls.append(self.server + each.get('href'))

	"""
	函数说明:获取章节内容
	Parameters:
		target - 下载连接(string)
	Returns:
		texts - 章节内容(string)
	Modify:
		2017-09-13
	"""
	def get_contents(self, target):
		req = requests.get(url = target)
		html = req.text
		bf = BeautifulSoup(html)
		texts = bf.find_all('div', class_ = 'showtxt')
		texts = texts[0].text.replace('\xa0'*8,'\n\n')
		return texts

	def search(self,search_name):
		self.author_search_list = [] 		#搜索用
		self.book_search_list = [] 			#搜索用
		self.url_search_list = []		 	#搜索用
		search_target = "https://so.biqusoso.com/s.php?ie=utf-8&siteid=biqukan.com&q=" + search_name
		req = requests.get(url = search_target)
		html = req.text
		bf = BeautifulSoup(html)
		# 搜索显示的作者名单
		author_texts = bf.find_all('span', class_ = 's4')
		for each in author_texts[1:]:
			self.author_search_list.append(each.string)
		# 搜索显示的作品名单
		book_texts = bf.find_all('span', class_ = 's2')
		for each in book_texts[1:]:
			self.book_search_list.append(each.string)
		# 给作品名单 + 作品
		for i in range(len(self.book_search_list)):
			self.book_search_list[i] += ("-" + self.author_search_list[i])
		# 搜索显示的作品url名单
		for each in book_texts[1:]:
			url = re.findall('''.*href="(.*)" target=.*''',str(each))
			self.url_search_list.extend(url)
		# 覆盖json数据
		self.url_data = {k: v for k, v in zip(self.book_search_list, self.url_search_list)}
		

		

	"""
	函数说明:将爬取的文章内容写入文件
	Parameters:
		name - 章节名称(string)
		path - 当前路径下,小说保存名称(string)
		text - 章节内容(string)
	Returns:
		无
	Modify:
		2017-09-13
	"""
	def writer(self, name, path, text):
		write_flag = True
		with open(path, 'a', encoding='utf-8') as f:
			f.write(name + '\n')
			f.writelines(text)
			f.write('\n\n')

		
	# 可视化界面
	def tkinter(self):
		window = tkinter.Tk()
		window.title("qqd小说爬虫")
		#画组件
		tkinter.Label(window, text="小说名称").grid(row=0)
		tkinter.Label(window, text="搜索").grid(row=1)
		tkinter.Label(window, text="下载路径：").grid(row=2)
		tkinter.Label(window, text="下载进度：").grid(row=3)
		#e1 = tkinter.Entry(window)
		e1 = tkinter.ttk.Combobox(window)
		e2 = tkinter.Entry(window)
		#e2 = tkinter.ttk.Combobox(window)
		e3 = tkinter.Entry(window)
		e4 = tkinter.Entry(window)
		e1.grid(row=0, column=1, padx=10, pady=5)
		e2.grid(row=1, column=1, padx=10, pady=5)
		e3.grid(row=2, column=1, padx=10, pady=5)
		e4.grid(row=3, column=1, padx=10, pady=5)
		e1["values"] = list(self.url_data.keys())
		#e1.set(list(self.url_data.keys())[0])
		e1.set("")
		#e2["values"] = ["暂无","暂无"]
		#e2.set("暂无")
		e2.insert(0, "")
		e3.insert(0, self.save_path)
		e4.insert(0, "尚未开始下载 ")

		# 防卡死，打包进线程
		def thread_it(func):
			# 创建
			t = threading.Thread(target=func) 
			# 守护 !!!
			t.setDaemon(True) 
			# 启动
			t.start()
			# 阻塞--卡死界面！
			# t.join()
			
        # 按钮"聚类算法"
		def button1():
			self.target = []
			self.names = []			#存放章节名
			self.urls = []			#存放章节链接
			self.nums = 0			#章节数
			self.book_name = str(e1.get())
			self.target = self.url_data[self.book_name]
			self.save_path = str(e3.get())
			mkdir(self.save_path)
			novel_download.get_download_url(self)

			#model.novel_save()
			book_save_path = self.save_path + f'{self.book_name}.txt'
			if os.path.exists(book_save_path):
				# 大于100kb的不要删，直接关闭
				if os.path.getsize(book_save_path)/1024 > 100 : 
					e4.delete(0, "end")
					e4.insert(0, f'已存在{self.book_name}，中止爬虫')
					return
				else:
					os.remove(book_save_path)
			e4.delete(0, "end")
			e4.insert(0, f'《{self.book_name}》开始下载')
			for i in range(self.nums):
				#time.sleep(1)
				novel_download.writer(self, self.names[i], self.save_path + f'{self.book_name}.txt', novel_download.get_contents(self, self.urls[i]))
				#sys.stdout.write("  已下载:%.3f%%" %  float(i/model.nums*100) + '\r')
				#sys.stdout.flush()	
				e4.delete(0, "end")
				e4.insert(0, "%.2f%%" %  float((i+1)/self.nums*100))
			print(f"{self.book_name}下载完成")
			

		# 按钮"搜索" 
		def button2():
			search_name = str(e2.get())
			novel_download.search(self, search_name)
			if len(self.url_data) == 0:
				e2.delete(0, "end")
				e2.insert(0, "未搜索到书名")
			e1["values"] = list(self.url_data.keys())
			e1.set(list(self.url_data.keys())[0])
			

		tkinter.Button(window, text="下载", width=10, command=lambda :thread_it(button1)).grid(row=4, column=1, sticky="w", padx=10, pady=5)
		tkinter.Button(window, text="搜索", width=10, command=button2).grid(row=4, column=0, sticky="w", padx=10, pady=5)
		tkinter.Button(window, text="退出", width=10, command=window.quit).grid(row=4, column=1, sticky="e", padx=10, pady=5)
		window.mainloop()

if __name__ == "__main__":
	#book_name = input()
	model = novel_download()
	model.tkinter()

	'''
	model.get_download_url()
	print(f'《{model.book_name}》开始下载：')
	for i in range(model.nums):
		model.writer(model.names[i], model.save_path + f'{model.book_name}.txt', model.get_contents(model.urls[i]))
		sys.stdout.write("  已下载:%.3f%%" %  float(i/model.nums*100) + '\r')
		sys.stdout.flush()
	print(f'《{model.book_name}》下载完成')
	'''



import os
import pandas as pd
import numpy as np

import matplotlib.dates as mdates
import matplotlib.pyplot as plt

import gensim
import lightgbm as lgb

from sklearn.model_selection import train_test_split
from sklearn.model_selection import GridSearchCV
from sklearn.metrics import roc_auc_score, accuracy_score, f1_score
from sklearn import preprocessing

from collections import Counter
from functools import reduce
import datetime


# 一些个人方法封装
class api():
    def __init__(self):
        pass
    
    def p(self, x):
        print(x)
        
    def os_analysis(self):
        from multiprocessing import cpu_count
        print("\nCPU的核数为：{}\n".format(cpu_count()))
    
    # 数据表情况观察
    def df_analysis(df):
        print("\n**************** DF Analysing ****************\n")
        print("shape of df is : {}".format(df.shape))
        print("")
        print("df memory : {:.2f} KB".format(df.memory_usage().sum()/(1024**1)))
        print("df memory : {:.2f} MB".format(df.memory_usage().sum()/(1024**2)))
        print("df memory : {:.2f} GB".format(df.memory_usage().sum()/(1024**3)))
        #print("info of df is :{}".format(df.info()))

        
        #print("\nhas null col : {}".format(x[x==True].index.tolist()))
        #print("notnull col : {}".format(x[x==False].index.tolist()))
        print("")
        tmp = pd.DataFrame({
            "null_ratio":df.isnull().sum(axis=0)/len(df),
            "null_num":df.isnull().sum(axis=0),
            "total_rows":df.shape[0],}).query('null_ratio > 0')  
        tmp["null_ratio"] = tmp["null_ratio"].apply(lambda x:str(round(100*x,2))+"%")
        print(tmp)
        print("")
        
        #print("\ndescribe of df is:\n{}\n".format(df.describe()))
        #print("head5 of df is :{}".format(df.head()))
        print("\n*************** Analysis Finished ************\n")

    # 做item2vec的序列处理，返回列表
    def get_item_query_list(self, df):
        def myreduce(list_input):
            return reduce(lambda x,y:str(x)+","+str(y), list_input)
        df["photo_id"] = df["photo_id"].astype(str)
        df = df.sort_values(by=["user_id","photo_rank"])
        item_query_df = df.groupby("user_id")["photo_id"].apply(myreduce)
        #print(item_query_df)
        query_list = []
        for i,c in item_query_df.items():
            if len(c) > 1:
                query_list.append(c.split(","))
            else:
                query_list.append(list(c))
        return query_list


    # 输入上述所出序列，输出物品向量模型和数据集
    def item2vec(self, list, min_count=1, size=3):
        model = gensim.models.Word2Vec(list, min_count=min_count, vector_size=size, seed=2021, workers=1)
        item2vec_df = pd.DataFrame(model.wv.vectors, index=model.wv.index_to_key)
        item2vec_df["photo_id"] = item2vec_df.index
        return model, item2vec_df


    # 最后的特征宽表中,photo_id换成item2vec，size是多少，就多几列
    def data_input_item2vec(self, data_input, item2vec_df):
        data_input["photo_id"] = data_input["photo_id"].astype(int)
        item2vec_df["photo_id"] = item2vec_df["photo_id"].astype(int)
        data_input = pd.merge(left=data_input, right=item2vec_df, how="left", on="photo_id")
        data_input.drop(columns="photo_id", inplace=True)
        return data_input

    '''
    # 为方便流式处理，定义模型方法
    def model_auto_load():
        import os
        if os.path.exists("lgb_model.txt"):
            # 模型加载
            gbm = lgb.Booster(model_file='lgb_model.txt')
        else:
            pass
    '''

    # 计算dtw值，没有除以k步
    def DTWDistance(self, s1, s2):
        DTW={}

        for i in range(len(s1)):
            DTW[(i, -1)] = float('inf')
        for i in range(len(s2)):
            DTW[(-1, i)] = float('inf')
        DTW[(-1, -1)] = 0

        for i in range(len(s1)):
            for j in range(len(s2)):
                dist= (s1[i]-s2[j])**2
                DTW[(i, j)] = dist + min(DTW[(i-1, j)],DTW[(i, j-1)], DTW[(i-1, j-1)])

        return sqrt(DTW[len(s1)-1, len(s2)-1])

    # 建立目录
    def mkdir(self, directory):
        if not os.path.exists(directory):
            os.makedirs(directory)
    # 清楚目录
    def rmdir(slef, directory):
        for root, dirs, files in os.walk(directory):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))

    # 网格搜索，参数优化
    def lgb_cv(self, x_train, y_train):
        cv_params = {
                'n_estimators':[100],
                'max_depth':range(2,15,1),
                'learning_rate':np.linspace(0.01,0.1,10),
                #'subsample':np.linspace(0.7,0.9,20),
                #'colsample_bytree':np.linspace(0.5,0.98,10),
                #'min_child_weight':range(1,9,1)
                }
        estimator = lgb.LGBMClassifier()

        gbm = GridSearchCV(estimator, cv_params, verbose=1, n_jobs=-1)
        gbm.fit(x_train, y_train.values.ravel(), verbose=2)
        print('Best parameters found by gr id search are:', gbm.best_params_)

    def plot(self, x, y, title="temp", label="deafault"):

        # 处理绘图格式
        plt.style.use("ggplot")
        # plt.style.use('fivethirtyeight')

        plt.figure(figsize=(20, 10))

        # 百分比处理
        def to_percent(temp, position):
            return "%.0f" % (100 * temp) + "%"

        plt.title(title)
        plt.plot(x, y, ls="-", color="darkorange", linewidth=1.5, label=label)

        plt.legend(loc="upper left", fontsize=12, frameon=False, ncol=1)
        plt.grid(axis="y", color="lightgray", linestyle="--", linewidth=0.5)

        extent = [
            mdates.date2num(min(x)), mdates.date2num(max(x)),
            #min(x), max(x),
            min(y), max(y),
        ]
        _, yv = np.meshgrid(np.linspace(0, 1, 210), np.linspace(0, 1, 90))
        plt.imshow(
            yv, cmap=plt.cm.Blues,
            origin="lower", alpha=0.6,
            aspect="auto", extent=extent,
        )
        plt.fill_between(x, y, max(y), color="white")
        plt.show() 







# -*- coding: utf-8 -*- 
import json
import sys
import getopt
from setuptools import setup
import time
import datetime
now = datetime.datetime.now()

version = str(now)
version = version.replace("-",".").replace(":",".").replace(" ",".")[0:16]


setup(
    name='zwdq',# 需要打包的名字,即本模块要发布的名字
    version = version ,# 版本
    description='个人接口', # 简要描述
    py_modules=['zwdq'],   # 需要打包的模块
    packages=['zwdq'],
    package_dir={'zwdq': 'src/zwdq'},
    author='zwdq', # 作者名
    author_email='zhuwudaoqin@gmail.com',   # 作者邮件
    requires=['numpy', 'pandas'], # 依赖包,如果没有,可以不要
)
